#include <opencv2/opencv.hpp>

template <typename T>
inline T getTranslationX(cv::Mat affineTransform)
{
    return affineTransform.at<T>(0, 2);
}

template <typename T>
inline T getTranslationY(cv::Mat affineTransform)
{
    return affineTransform.at<T>(1, 2);
}

template <typename T>
inline T getScaleX(cv::Mat affineTransform)
{
    return cv::norm(cv::Point_<T>(affineTransform.at<T>(0, 0), affineTransform.at<T>(1, 0)));
}

template <typename T>
inline T getScaleY(cv::Mat affineTransform)
{
    return cv::norm(cv::Point_<T>(affineTransform.at<T>(0, 1), affineTransform.at<T>(1, 1)));
}

template <typename T>
inline T getRotation(cv::Mat affineTransform)
{
    T sx = getScaleX<T>(affineTransform);
    T sy = getScaleY<T>(affineTransform);
    if (sx < 0.0001 || sy < 0.0001) return 0;
    return atan2(affineTransform.at<T>(0, 1) / sy, affineTransform.at<T>(0, 0) / sx);
}

template <typename T>
inline cv::Mat combineAffineTransforms(cv::Mat first, cv::Mat second)
{
    cv::Mat A = cv::Mat_<T>::eye(3, 3);
    cv::Mat B = cv::Mat_<T>::eye(3, 3);
    first.copyTo(A(cv::Rect(0, 0, 3, 2)));
    second.copyTo(B(cv::Rect(0, 0, 3, 2)));
    cv::Mat C = B * A;
    return C(cv::Rect(0, 0, 3, 2));
}

template <typename T>
inline cv::Mat getAffineTransform(T scaleX, T scaleY, T rotation, T translationX, T translationY)
{
    cv::Mat warp_matrix = cv::Mat_<T>::zeros(2, 3);
    warp_matrix.at<double>(0, 0) = std::cos(rotation) * scaleX;
    warp_matrix.at<double>(0, 1) = -std::sin(rotation) * scaleY;
    warp_matrix.at<double>(1, 0) = std::sin(rotation) * scaleX;
    warp_matrix.at<double>(1, 1) = std::cos(rotation) * scaleY;
    warp_matrix.at<double>(0, 2) = translationX;
    warp_matrix.at<double>(1, 2) = translationY;
    return warp_matrix;
}
