#include <opencv2/opencv.hpp>

#include "affineTransformTools.h"

int main()
{
    cv::VideoCapture video("../record2.avi"); 
    double fps = video.get(cv::CAP_PROP_FPS);
    cv::Mat frame, prev_gray, gray;
    video >> frame;
    cv::cvtColor(frame, prev_gray, cv::COLOR_BGR2GRAY);

    cv::VideoWriter out("../record3.avi",cv::VideoWriter::fourcc('M','J','P','G'),fps, frame.size());

    double x = 0.0;
    double y = 0.0;
    double f = 0.0;

    for (;;) {

        cv::cvtColor(frame, gray, cv::COLOR_BGR2GRAY);

        std::vector<cv::Point2f> prev_points;
        cv::goodFeaturesToTrack(prev_gray, prev_points, 200, 0.01, 30, cv::noArray(), 3);

        std::vector<cv::Point2f> points;
        std::vector<uchar> status;
        std::vector<float> err;
        cv::calcOpticalFlowPyrLK(prev_gray, gray, prev_points, points, status, err);

        std::vector<cv::Point2f> selected_prev_points, selected_points;
        for (size_t i = 0; i < status.size(); i++) {
            if (status[i] != 1) continue;
            selected_prev_points.push_back(prev_points[i]);
            selected_points.push_back(points[i]);
        }

        cv::Mat warp_matrix = cv::estimateAffinePartial2D(selected_prev_points, selected_points, cv::noArray(), cv::LMEDS);

        double dx = getTranslationX<double>(warp_matrix);
        double dy = getTranslationY<double>(warp_matrix);
        double df = getRotation<double>(warp_matrix);

        x += dx;
        y += dy;
        f += df;

        cv::Mat final_warp_matrix = getAffineTransform(1.0, 1.0, f, -x, -y);

        cv::Mat stabilized(frame.size(),frame.type());
        cv::warpAffine(frame, stabilized, final_warp_matrix, frame.size());

        cv::Mat combined;
        cv::hconcat(std::vector<cv::Mat> { frame, stabilized }, combined);
        cv::imshow("video",combined);
        int key = cv::waitKey(1);
        if (key == 27) break;

        out << stabilized;

        video >> frame;
        if (frame.empty()) break;

        prev_gray = gray.clone();
    }
    
    out.release();
    video.release();
    cv::destroyAllWindows();
    return 0;
}

