import numpy as np
import cv2
import yaml
import itertools

def saveCalib(fname,data):
    with open(fname, "w") as f:
        yaml.dump(data, f)
        
    print(fname,'saved')
        
def loadCalib(fname):
    with open(fname, "r") as f:
        data = yaml.load(f, Loader=yaml.Loader)
        
    print(fname,'saved')
    return data

camera = cv2.VideoCapture(0,cv2.CAP_DSHOW)

nx = 7 #number of inside corners in x
ny = 7 #number of inside corners in y
objp = np.array([(j,i,0) for i,j in itertools.product(range(nx),range(ny))], np.float32)
criteria = (cv2.TERM_CRITERIA_EPS + cv2.TERM_CRITERIA_MAX_ITER, 30, 0.001)

objpoints = []
points = []
mtx = None

while True:
    hasFrame, frame = camera.read()
    if not hasFrame:
        break
    
    cv2.imshow('camera',frame)    
    if not mtx is None:
        undistorted = cv2.undistort(frame, mtx, dist, None, newcameramtx)
        cv2.imshow('undistorted',undistorted)
        
    key = cv2.waitKey(10)
    if key == 27:
        break
    elif key == ord('c'):

        # add next sample 
        gray = cv2.cvtColor(frame, cv2.COLOR_BGR2GRAY)
        found, corners = cv2.findChessboardCorners(gray, (nx,ny), cv2.CALIB_CB_ADAPTIVE_THRESH+cv2.CALIB_CB_FAST_CHECK+cv2.CALIB_CB_NORMALIZE_IMAGE)
        if found == True:
            dims = gray.shape[::-1]
            
            corners2 = cv2.cornerSubPix(gray,corners,(11,11),(-1,-1),criteria) # refinement

            cv2.drawChessboardCorners(frame, (nx, ny), corners2, found)
            cv2.imshow('chessboardCorners',frame)
            
            objpoints.append(objp)
            points.append(corners2)
            
            print('calibrating')
            ret, mtx, dist, rvecs, tvecs = cv2.calibrateCamera(objpoints, points, dims, None, None)
            newcameramtx, roi = cv2.getOptimalNewCameraMatrix(mtx, dist, dims, 1, dims)
            print('calibrated')
            
        else:
            print("calibration chessboard not detected")
            #break

    elif key == ord('s'):
        if not mtx is None:
            saveCalib('calib.yaml', [mtx, dist, dims])
            print('calibration saved')
        
    elif key == ord('l'):
        try:
            mtx, dist, dims = loadCalib('calib.yaml')
            newcameramtx, roi = cv2.getOptimalNewCameraMatrix(mtx, dist, dims, 1, dims)
            print('calibration loaded')
        except:
            pass
            
    elif key == ord('r'):
        objpoints = []
        points = []
        mtx = None
        print('calibration reset')

camera.release()
cv2.destroyAllWindows()
